@@ -1,40 +1,53 @@
-extern void inituse(),readstructs(),initlog();
-void usage(),quit();
+/* GLOBAL externs file for kdrill!!
+ * Hopefully, the only stuff that is in here, is stuff that should REALLY
+ * be globally visible, not just any stuff that happens to be exported
+ * by a particular file/module
+ */
+
+extern Atom delete_message,wm_message;
+
+
+
 
-extern voidMakeWidgets();
-extern void initstuffs(int *,char **);
 
 extern struct translationstruct *translations[MAXKANJIALLOWED];
-extern Boolean useKanji[MAXKANJIALLOWED];
 
-extern int lowest,highest,numberofkanji;	/* keep track of how many we have */
+extern int lowestkanji,highestkanji,numberofkanji;/* keep track of how many we have */
 extern int lowfrequency,highfrequency;		/* for frequency restrictions */
-extern char *dictname,*usefile,kanjifontname[];
 
-extern void Beep(),MakeGradeSelect(),MakeFrequency();
-extern void printgrades(), parsegrades(char *);
+extern char *usefile;
+
+void usage(),quit();
+extern void Beep();	/* nice generic routine that respects whether
+			 * player wants quiet or not
+			 */
+
+extern void MakeFrequency();
+
+extern void SearchCallback(Widget,XtPointer,XtPointer);
+
 
 /* widget operators.. */
-extern void UnreverseButton(Widget);
 extern int GetWidgetNumberval(Widget);
-extern void ChangeMode(Widget,XtPointer,XtPointer);
+extern void UnreverseButton(Widget);
+extern void SetWidgetNumberval(Widget,int);
+extern void SetWidgetHexval(Widget,int);
 extern void ReverseButton(Widget);
 
-/* other functions */
-extern int xtoi(),getline();
+extern int FindIndex(char *);
 
-/* callbacks */
-extern void englishcallback(Widget,XtPointer,XtPointer),
-            kanjicallback(Widget,XtPointer,XtPointer),
-	    cheatcallback(Widget,XtPointer,XtPointer),
-	    usefilecallback(Widget,XtPointer,XtPointer);
-
-extern void SetupGuess(),CountKanji();
-extern void DescribeCurrent(int);
 
 extern XFontStruct *largekfont;
 extern XFontStruct *smallkfont;
-extern XFontStruct *fixedfont;
+extern XFontStruct *englishfont;
+
+/* other "usefull" generic functions */
+extern int xtoi(),getline();
+extern void setstatus(char *);
+
+
+/* callbacks have been moved to their specific .h files */
+
 
 extern XtAppContext Context;
 
@@ -43,32 +56,35 @@ extern Display	*display;
 extern Window	mainwindow,rootwindow;
 extern GC	gc,cleargc;
 
-/* buttons n widgets */
+/* buttons 'n' widgets */
+extern Widget toplevel,form;
+extern Widget options_popup;
 extern Widget frequencyForm,frequencyHigh,frequencyLow;
 extern Widget buttonform,englishform,kanjiform;
-extern Widget gradelevelForm,gradeButtons[];
-extern Widget toplevel,form,quitbutton,statusline;
-extern Widget usefilebutton;
 extern Widget kanjimode,englishmode,togglekana;
 extern Widget englishWidget[NUMBEROFCHOICES],kanjiWidget[NUMBEROFCHOICES];
+extern Widget gradelevelForm,gradeButtons[];
+extern Widget usefilebutton,orderbutton;
+
 
-extern void setstatus(char *);
 /* Accelerator functions.. */
 extern void UpdateFrequency(Widget,XEvent *,String *,Cardinal *);
-extern void Guessvalue(Widget,XEvent *,String *,Cardinal *);
+extern void DoFind(Widget,XEvent *,String *,Cardinal *);
 
 /* misc resource-level stuffs... */
 extern void GetXtString(char *, char *, char *);
 extern int GetXtNumber(char *,char *);
 extern Boolean GetXtBoolean(char *, char *);
 
+/* These flags indicate current settings of the game */
 extern Boolean useUsefile; /*	we Always read in a usefile, but
 				this says if we actually use it */
 extern Boolean doBell,showinorder;
 extern Boolean showEnglish;
 extern Boolean switchKanaEnglish;
 extern int gradelevelflags;
+
+#ifdef OLDLOG
 extern FILE *logfile;
-
-extern int Kanji2English;
+#endif
 
