@@ -1,24 +1,14 @@
 /* game.c:
- *	Externally used routines here:
- * ChangeMode()		-- english<->kanji<->kana
- * CountKanji()		-- returns number of active kanji, given select parms
- * SetupGuess()		-- called each "round" to set things up
- * UseThisKanji()	-- Returns Boolean if we are allowed to use this kanji
- * cheatcallback()	-- callback for cheat button
- * englishcallback()	-- callback for english buttons
- * kanjicallback()	-- callback for kanji buttons
- * picktruevalue()	-- sets up which kanji is the right value
- * pickkanji()		-- picks four kanji to guess
- * printenglish()	-- sets english labels active/inactive, etc
- * printkanji()		-- sets kanji labels active/inactive, etc
- * setstatus(char *)	-- sets status bar
- * guessvalue(int)	-- given the pick of the user, will say if was right
- * Guessvalue()		-- accelerator wrapper for guessvalue()
+ *	This file should contail all the internal game mechanisms;
+ *	things that deal with the way things play, not how the
+ *	windows look.
  */
  
 
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdlib.h>	/* This SHOULD define lrand48..
+			 *	but doesn't for solaris. go figure
+			 */
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
@@ -28,10 +18,17 @@
 
 #include "defs.h"
 #include "externs.h"
+#include "widgets.h"
+#include "grades.h"
+#include "init.h"
 
-
+int lastpicked=0;/* we can't "really" auto-initialize this...
+		  * set to zero so we can set it
+		  * properly later
+		  */
 int values[NUMBEROFCHOICES];
-int truevalue;
+int truevalue=0;
+int numberincorrect=0;
 
 int Kanji2English=GUESSKANJI;
 
@@ -39,22 +36,11 @@ Boolean doBell,showinorder,showEnglish;
 Boolean useUsefile;
 Boolean switchKanaEnglish;
 
-extern Display *display;
-extern Window mainwindow;
 
-int englishwidth=7;
-int englishheight=11;
+int englishwidth=7;	/* unused? */
+int englishheight=11;	/* unused? */
 
-/* setstatus:
- *  sets label for main status bar widget.
- *	( the long one at the bottom)
- */
-void setstatus(char *s)
-{
-	XtVaSetValues(statusline,
-		      XtNlabel,s,
-		      NULL);
-}
+void SetupGuess();
 
 /*
 *UseThisKanji()
@@ -76,12 +62,18 @@ Boolean UseThisKanji(int kanjinum)
 
 
 	/* check against frequency limits */
-	if(kanji->frequency==0)
-		if(lowfrequency != 0) return False;
+	if(kanji->frequency==0) {
+		/* No frequency means REALLY LOW frequency,
+		 * therefore if there's ANY limit, dont use
+		 */
+		if(lowfrequency != 0)
+			return False;
+	} else {
+		/* "1" is highest frequency!! */
+		if(kanji->frequency < highfrequency) return False;
+	}
 
-	   /* "1" is highest frequency!! */
-	if(kanji->frequency < highfrequency) return False;
-	if(lowfrequency >0){
+	if(lowfrequency != 0){
 		if(kanji->frequency >lowfrequency)
 			return False;
 	}
@@ -125,19 +117,25 @@ Boolean UseThisKanji(int kanjinum)
  *	when we change kana/english display.
  *	 (the secnd being because kanjidic does not always have
  *	 english and/or kana listings
+ *
+ *   This counts the number of usable kanji.
+ *   It ALSO has to update the "numberincorrect" counter;
  */
 void CountKanji(){
 	int counter;
 	numberofkanji=0;
-	for(counter=lowest;counter <=highest;counter++){
+	numberincorrect=0;
+	for(counter=lowestkanji;counter <=highestkanji;counter++){
 		if(UseThisKanji(counter)){
 			numberofkanji++;
+			if(translations[counter]->incorrect >0)
+				numberincorrect++;
 		}
 	}
 }
 
 /* pickkanji:
- *   picks an acceptably random kanji char.
+ *   picks an acceptably random kanji index.
  *	NOTE THAT THIS DEPENDS ON "numberofkanji" being correct!
  */
 
@@ -147,7 +145,7 @@ int pickkanji(){
 
 	rand_kanji = lrand48()%numberofkanji;
 
-	for(count=lowest;count<=highest;count++) {
+	for(count=lowestkanji;count<=highestkanji;count++) {
 		if(UseThisKanji(count)) {
 		      rand_kanji--;
 		      if(rand_kanji <0)
@@ -161,113 +159,222 @@ int pickkanji(){
 	exit(0);
 
 }
+
+/* pickincorrect:
+ *	pick a kanji we have missed previously.
+ *	start at first incorect kanji after the last one picked.
+ *	This hopefully ensures that we don't replay the same incorrect
+ *	one too often.
+ */
+int pickincorrect()
+{
+	int currentpick=lastpicked;
+	if(numberincorrect==0){
+		puts("Internal Error.. pickincorrect called when no incorrect stored");
+		return pickkanji();
+	}
+	do {
+		do {
+			currentpick++;
+			if(currentpick >highestkanji)
+				currentpick=lowestkanji;
+		} while(!UseThisKanji(currentpick));
+	} while (translations[currentpick]->incorrect==0);
+	setstatus("You missed this one before");
+	return currentpick;
+}
+
 /*picktruevalue()
- *	Similar to pickkanji(),
- *	except that this has special handling for
- *	picking the "truevalue" (for going in order! )
- *	Note: this will either loop forever, or return same value
- *	   if there aren't enough valid kanji to use.
- *	   This should never happen, but....
+ *	Similar to pickkanji().
+ *	This calls pickkanji() to generate random kanji when
+ *	desired. 
+ *	Also tries to go back to missed kanji, ONLY when
+ *	"random" order is in effect.
+ *
+ *	Tries to make sure same kanji isn't called in a row.
+ *
+ *	Sets "lastpicked" global, AFTER we have picked a new value.
+ *	As soon as we have picked a value, it becomes the "lastpicked".
+ *
+ *	"lastpicked" is used by the "Back" command, outside this file.
  */
 int picktruevalue(){
-	static int lastpicked=0;/* we can't "really" auto-initialize this...
-				 * set to zero so we can set it
-				 * properly lower down
-				 */
-	
-	if(!showinorder)
-		return (pickkanji());
-	if(lastpicked==0)
-		lastpicked = lowest;
-	else
-		lastpicked++;
-	while(!UseThisKanji(lastpicked)){
-		lastpicked++;
+	int currentpick;
 
-		if(lastpicked >highest)
-			lastpicked = lowest;
+
+	if(!showinorder){
+		if(numberincorrect>0){
+			if(lrand48()%REPEATFRACTION == 0){
+				currentpick = pickincorrect();
+			}
+			else do {
+				currentpick = pickkanji();
+			} while (currentpick == lastpicked);
+			
+		} else 
+		do {
+			currentpick = pickkanji();
+		} while (currentpick == lastpicked);
+
+	} else {
+		/* ASSUME this loop will terminate. If there is
+		 * something wrong with program setup, it will
+		 * not. This is why there are checks for a minimum
+		 * number of kanji here and there.
+		 */
+		currentpick = lastpicked;
+		currentpick++;
+		while(!UseThisKanji(currentpick)){
+			currentpick++;
+
+			if(currentpick >highestkanji)
+				currentpick = lowestkanji;
+		}
 	}
-	return lastpicked;
-}
+
+	return currentpick;
+} 
+
+
 /*guessvalue(int)
  *	passed number by EITHER kanji button or english button,
  *	we don't care.
- *	Set status bar to correct/incorrect. beeps if incorrect
- *	returns True if correct, False otherwise.
- */
+ *	Set status bar to correct/incorrect. Beeps if incorrect
+ *	Also sets "incorrect" counter in the translations[].
+ *	
+*/
+/* note that "-1" on guess means "you lose: ran out of time" */
+
 void guessvalue(int guess){
+	struct translationstruct *kanjiP;
+
+	kanjiP = translations[values[guess]];
 	if(guess == truevalue){
-		setstatus("Correct!");
+		setstatus("Correct! Now try THIS one...");
+		switch(kanjiP->incorrect){
+			case 0:
+				break;
+			case 1:
+				kanjiP->incorrect -= 1;
+				numberincorrect -= 1;
+				break;
+			default:
+				kanjiP->incorrect -= 1;
+		}
 		SetupGuess();
 		return;
 	} 
+
+	/* YOU GOT IT **WWRRROONNGGG!!!!** */
+	/* we count off TWICE;
+	 *  once for quiz meaning you missed,
+	 *  and once for the incorrect kanji you thought it was
+	 */
 	Beep();
 	setstatus("Incorrect.");
+#ifdef OLDLOG
 	if(logfile != NULL){
 		fprintf(logfile,"%x -> %x\n",values[truevalue],values[guess]);
 	}
-}
-/* Guessvalue:
- *	accelerator function so we can guess with 1,2,3,4 keys
- */
-void Guessvalue(Widget w,XEvent *event,String *params,Cardinal *num_parags){
-	int value;
-	value = atoi(*params);
-	guessvalue(value);
+#endif
+	if(guess != -1) {
+		kanjiP->incorrect += REPEATTIMES;
+		if(kanjiP->incorrect ==REPEATTIMES)
+			numberincorrect+=1;
+	}
+
+	kanjiP = translations[values[truevalue]];
+	kanjiP->incorrect += REPEATTIMES;
+	if(kanjiP->incorrect ==REPEATTIMES)
+		numberincorrect+=1;
+	
 }
 
-/* printkanji:
+
+
+void printkanji(Widget w,unsigned knum){
+	XChar2b onecharstring[2];
+	onecharstring[1].byte1 = onecharstring[1].byte2 = 0;
+
+	onecharstring[0].byte1 = (knum & 0xff00)>>8;
+	onecharstring[0].byte2 = (knum & 0x00ff);
+
+	XtVaSetValues(w,XtNlabel,onecharstring,NULL);
+
+}
+
+/* printallkanji:
  *	updates all the kanji buttons.
  */
 
-void printkanji(){
-	int i,knum;
-	XChar2b onecharstring[2];
+void printallkanji(){
+	int i;
+	unsigned knum;
 
 	for(i=0;i<NUMBEROFCHOICES;i++){
 		UnreverseButton(kanjiWidget[i]);
 	}
-	onecharstring[1].byte1 = onecharstring[1].byte2 = 0;
 
 	if(Kanji2English == GUESSKANJI){
 		knum = values[truevalue];
-		onecharstring[0].byte1 = (knum & 0xff00)>>8;
-		onecharstring[0].byte2 = (knum & 0x00ff);
+		printkanji(kanjiWidget[0],knum);
 
-		XtVaSetValues(kanjiWidget[0],XtNlabel,onecharstring,NULL);
 		for(i=1;i<NUMBEROFCHOICES;i++){
-			XtVaSetValues(kanjiWidget[i],XtNlabel,"  ",NULL);
+			printkanji(kanjiWidget[i],(unsigned) 0);
+			/*XtVaSetValues(kanjiWidget[i],
+			      XtNsensitive,False,
+			      NULL);
+			  */
+
 		}
+
+		/* never more than 4 ON, I hope.. */
+		if(translations[knum]->ON_extra != 0){
+			int onloop=0;
+			XChar2b *on = translations[knum]->ON_extra;
+
+#ifdef SINGLE_WINDOWS
+			while(on[onloop].byte1 !=0){
+				if(onloop==NUMBEROFCHOICES -1) break;
+				
+				printkanji(kanjiWidget[onloop+1],
+					   ((on[onloop].byte1 <<8) +
+					    on[onloop].byte2) );
+				onloop++;
+			}
+#endif /* SINGLE_WINDOWS */
+#ifdef USE_OKU
+ 			XtVaSetValues(ONWidget,XtNlabel,on,NULL);
+#endif
+		}
+#ifdef USE_OKU
+		else {
+			printkanji(ONWidget,(unsigned) 0 );
+		}
+#endif
+		
 		return;
 	}
 	/* else */
 	for(i=0;i<NUMBEROFCHOICES;i++) {
-		knum = values[i];
-
-		onecharstring[0].byte1 = (knum & 0xff00)>>8;
-		onecharstring[0].byte2 = (knum & 0x00ff);
-
-
-		XtVaSetValues(kanjiWidget[i],
-			      XtNlabel,onecharstring,
-			      XtNsensitive,True,
-			      NULL);
-
+		printkanji(kanjiWidget[i],values[i]);
+		XtSetSensitive(kanjiWidget[i],True);
 	}
 }
 
-/* SetWidgetLabel:
+
+/* printenglish
  *	Convenience function to
- *	set label of four lines of english/kana.
+ *	set label of  of english/kana.
  *	Will change fonts, dependant on "showEnglish"
  */
-void SetWidgetLabel(Widget widget,int Tnum)
+void printenglish(Widget widget,unsigned Tnum)
 {
 
 	if(showEnglish == True){
 		XtVaSetValues(widget,
 			     XtNencoding,XawTextEncoding8bit,
-			     XtNfont,fixedfont,
+			     XtNfont,englishfont,
 			     XtNlabel,translations[Tnum]->english,
 			     NULL);
 	} else {
@@ -280,13 +387,16 @@ void SetWidgetLabel(Widget widget,int Tnum)
 			      NULL);
 	}
 }
-/* printenglish()
- *  similar to printkanji()
+
+
+
+/* printallenglish()
+ *  similar to printallkanji()
  *  updates all "english" labels...
  *  EXCEPT: sometimes we want then to print kana! :-) so we use
- *  SetWidgetLabel() to do the correct type
+ *  printenglish() to do the correct type
  */
-void printenglish(){
+void printallenglish(){
 	int i;
 
 	for(i=0;i<NUMBEROFCHOICES;i++){
@@ -294,23 +404,26 @@ void printenglish(){
 	}
 	/* we must be guessing which character for one meaning */
 	if(Kanji2English == GUESSMEANING){
-		SetWidgetLabel(englishWidget[0],values[truevalue]);
+		printenglish(englishWidget[0],values[truevalue]);
 		for(i=1;i<NUMBEROFCHOICES;i++){
 			XtVaSetValues(englishWidget[i],
-			  XtNlabel,"  ",XtNsensitive,False,NULL);
+			  XtNlabel,"  ",
+			  /*XtNsensitive,False,*/
+			  NULL);
 		}
 		return;
 	}
 	/* ELSE */
 	/* we have all FOUR  active for picking*/
 	for(i=0;i<NUMBEROFCHOICES;i++){
-		XtVaSetValues(englishWidget[i],XtNsensitive,True,NULL);
-		SetWidgetLabel(englishWidget[i],values[i]);
+		XtSetSensitive(englishWidget[i],True);
+		printenglish(englishWidget[i],values[i]);
 	}
 }
 
 
 
+
 /* SetupGuess
  *		"MAIN LOOP" OF PROGRAM
  ******************************************************
@@ -329,14 +442,16 @@ void SetupGuess()
 {
 	Boolean doloop;
 	int valuecount;
-	static int savedgrade = ALLGRADES;
+
+	lastpicked = values[truevalue];
 
 	truevalue = lrand48() %NUMBEROFCHOICES;
 	for(valuecount=0;valuecount<NUMBEROFCHOICES;valuecount++){
-		if(valuecount == truevalue)
+		if(valuecount == truevalue){
 			values[valuecount] = picktruevalue();
-		else
+		} else {
 			values[valuecount] = pickkanji();
+		}
 	}
 
 	/* now weed out duplicates..
@@ -366,95 +481,11 @@ void SetupGuess()
 
 	DescribeCurrent(values[truevalue]);
 
-	printkanji();	/* update labels */
-	printenglish(); /* likewise      */
+	printallkanji();	/* update labels */
+	printallenglish(); /* likewise      */
 	printgrades();	/* stupid buttons needs updating manually,
 			 * EVERY durn time. arrg */
 
-	/* This should really be here, but it works best here... */
-	switch(Kanji2English){
-		case GUESSKANJI:
-			XtVaSetValues(kanjimode,XtNsensitive,False,NULL);
-			XtVaSetValues(englishmode,XtNsensitive,True,NULL);
-			break;
-		case GUESSMEANING:
-			XtVaSetValues(kanjimode,XtNsensitive,True,NULL);
-			XtVaSetValues(englishmode,XtNsensitive,False,NULL);
-			break;
-	}
 }
 
 
-/* englishcallback:
- *	Handles clicking on "english" widgets.
- *	Only really neccessary if we are guessing the meaning of a
- *	kanji character
- */
-void englishcallback(Widget w,XtPointer data,XtPointer calldata)
-{
-	if(Kanji2English!= GUESSKANJI)
-		return;
-	guessvalue((int) data);
-}
-
-/*
- *handles top four kanji buttons at top
- */
-void kanjicallback(Widget w,XtPointer data,XtPointer calldata)
-{
-	if(Kanji2English != GUESSMEANING)
-		return;
-	guessvalue((int) data);
-}
-
-void cheatcallback(Widget w,XtPointer data,XtPointer calldata){
-	if(Kanji2English == GUESSKANJI){
-		ReverseButton(englishWidget[truevalue]);
-	} else {
-		ReverseButton(kanjiWidget[truevalue]);
-	}
-}
-
-/* changemode:
- *   change from guessing kanji to guessing english, or vica verse,
- *	by clicking on the appropriate button
- *
- *   Also handles changing from english <->kana display,
- *   calling CountKanji(), because we have to .
- */
-void ChangeMode(Widget w,XtPointer data,XtPointer calldata)
-{
-	/* whichbutton is
-	 * EITHER
-	 *    4kanji<->4english button,
-	 * OR
-	 *    kana<->english
-	 */
-	int whichbutton = (int) data;
-#ifdef DEBUG
-	printf("button mode change: %d\n",whichbutton);
-#endif
-	switch(whichbutton){
-		case GUESSMEANING:
-		case GUESSKANJI:
-			if(Kanji2English == whichbutton)
-				return;
-			Kanji2English = whichbutton;
-			SetupGuess();
-			break;
-		case TOGGLEKANA:
-#ifdef DEBUG
-			puts("We toggle kana/english now");
-#endif
-			showEnglish = !showEnglish;
-			if(!switchKanaEnglish){
-				CountKanji();
-				SetupGuess();
-			} else {
-				printenglish();
-			}
-			
-			break;
-	}
-}
-
