@@ -13,7 +13,9 @@
 #include <Xaw/Form.h>
 #include "defs.h"
 #include "externs.h"
-
+#include "game.h"
+#include "grades.h"
+#include "options.h"
 
 Widget gradelevelForm;
 
@@ -21,6 +23,9 @@ Widget gradelevelForm;
 Widget gradeButtons[8];
 int gradelevelflags;
 
+
+
+
 /* normally, I would make this inline, but
  * " grades.c:20: warning: ANSI does not permit the keyword `inline' "
  * Hurmph....
@@ -90,6 +95,7 @@ void parsegrades(char *gradestring){
  */
 void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 	char statusbuff[100];
+	char statusprefix[100];
 	int grademask;
 	grademask = 1 << (int) data;
 	
@@ -117,20 +123,32 @@ void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
 						
 					}
 					UnreverseButton(w);
+					if((int) data <6)
+						sprintf(statusprefix,"Grade %d unset: ",(int) data);
+					else
+						sprintf(statusprefix,"Grade + unset: ");
 				}
+				/* CountKanji already called above */
                    	} else {
 			/* nope.. turn on grade level */
 				ReverseButton(w);
 				gradelevelflags |= grademask;
+					if((int) data <6)
+						sprintf(statusprefix,"Grade %d set: ",(int) data);
+					else
+						sprintf(statusprefix,"Grade + set: ");
 			}
+			CountKanji();
 			break;
 
 		case 0:/* "All" button */
 			gradelevelflags = ALLGRADES;
 			printgrades();
+			sprintf(statusprefix,"All grades selected: ");
+			CountKanji();
 			break;
 	}
-	sprintf(statusbuff,"%d characters active",numberofkanji);
+	sprintf(statusbuff,"%s%d kanji active",statusprefix,numberofkanji);
 	setstatus(statusbuff);
 }
 
@@ -142,9 +160,11 @@ void MakeGradeSelect(){
 	
 	int widgetcount;
 	gradelevelForm = XtVaCreateManagedWidget(
-			"gradedisplay",formWidgetClass,buttonform,
+			"gradedisplay",formWidgetClass,optionsform,
 			XtNhorizDistance,GRADEDISTANCE,
 			XtNfromHoriz,usefilebutton,
+			XtNfromHoriz,togglekana,
+			XtNfromHoriz,englishmode,
 			XtNvertDistance,20,
 			XtNresize,False,
 			NULL);
