@@ -14,6 +14,11 @@
 
 #include "defs.h"
 #include "externs.h"
+#include "init.h"
+#include "readfile.h"
+#include "game.h"
+#include "grades.h"
+
 #include "patchlevel.h"
 
 static char *version = VERSION;
@@ -24,10 +29,6 @@ Window mainwindow,rootwindow;
 GC gc,cleargc;
 XtAppContext Context;
 
-char *dictname=NULL;
-char *usefile=NULL;
-FILE *logfile;
-
 XChar2b *kstring;
 XChar2b onecharstring;
 
@@ -35,6 +36,29 @@ unsigned long white,black;
 
 
 
+
+/* REAL  OS's don't need this. 
+ * But because I'm such a nice guy, and recognise that there
+ * are screwey OS's out there...
+ */
+void clean_up_structs(){
+	int i;
+	for(i=lowestkanji; i<=highestkanji;i++){
+		struct translationstruct *T;
+		if(translations[i] == NULL)
+			continue;
+		T = translations[i];
+
+		if(T->english != NULL)
+			free(T->english);
+		/* for some reason, 0 is better than NULL */
+		if(T->pronunciation != 0) 
+			free (T->pronunciation);
+		if(T->ON_extra != 0)
+			free(T->pronunciation);
+	}
+}
+
 /* quitbutton, of course */
 void quit(Widget w,XtPointer data,XtPointer calldata)
 {
@@ -42,8 +66,12 @@ void quit(Widget w,XtPointer data,XtPointer calldata)
 	puts("quitting?");
 #endif
 	XtCloseDisplay(display);
+#ifdef OLDLOG
 	if(logfile != NULL)
 		fclose(logfile);
+#endif
+	/* I supose we'll be nice an' clean up memory somewhat.. */
+	clean_up_structs();
 	exit(0);
 }
 
@@ -58,7 +86,7 @@ void usage(){
 	puts("  -nousefile                   Do not abridge initially");
 	puts("  -dictfile dictfilename       Changes dictionary file from");
 	puts("                               \"kanjidic\" to some other file");
-	puts("  -font fontname               Changes english font");
+	puts("  -englishfont fontname        Changes english button font");
 	puts("  -kanjifont Kanjifontname     Changes LARGE kanji font");
 	puts("  -smallkanji Kanjifontname    Changes small kanji font");
 	puts("  -noBell                      Turns off beep for incorrect answer");
@@ -72,7 +100,7 @@ void usage(){
 	puts("   -lowfrequency               Lowest frequency kanji you wish to see.");
 	puts("   -highfrequency              Highest frequency kanji you wish to see");
 	puts("                  (\"1\" is HIGH, \"2000\" is low)");
-	puts("   -logfile filename           Store wrong answers in filename");
+	puts("   -logfile filename           Set logfile to non-standard name");
 	puts("");
 	puts("       The above options can also be set as resources,");
 	puts("       with the same name as their optionflag");
@@ -80,11 +108,16 @@ void usage(){
 	exit(0);
 }
 
-
+/* Generic Beep routine that we can call from anywhere.
+ * This way we don't have to check for enabled bell every time in the code.
+ * We can just put Beep() whereever we feel like it.
+ */
 void Beep(){
 	if(doBell)
 		XBell(display,100);
 }
+
+
 /* GetXtNumber(), GetXtBoolean(), GetXtString():
  *	Not the "best way" to get resources...
  *	supposed to read them all in at one go.
@@ -99,7 +132,7 @@ void Beep(){
  *	Destination must be MALLOCED ARRAY!!!
  *
  *	Default value of string is "NOT SET".
- *	This is used by initlog()
+ *	This is used by initlog() [which is not used any more]
  *	This should actually be encountered. I provide default values for
  *	all resource strings used.
  */
@@ -167,11 +200,13 @@ Boolean GetXtBoolean(char *name,char *class)
 }
 
 
+
+/*
+ * Welcome, Welcome, Welcome!
+ */
 int main(int argc,char *argv[])
 {
 
-	char usefilename[100],dict[200];
-
 	puts("kdrill: Starting up... please wait a while.");
 	fflush(stdout);
 
@@ -179,24 +214,16 @@ int main(int argc,char *argv[])
 	/* also, resources get called */
 	initstuffs(&argc,argv);
 
+ /*	ReverseButton(form);*/
 
-	GetXtString("usefile","Usefile",usefilename);
-	GetXtString("dictfile","Dictfile",dict);
-	dictname = dict;
-	usefile = usefilename;
-
-#ifdef DEBUG
-	puts("");
-	printf("usefile from resources is \"%s\"\n",usefilename);
-	printf("dictfile from resources is\" %s\"\n",dictname);
-#endif
 
 	/* check for abridgements of the dictionary... */
-	inituse();
+	initusefile();
 	/* and actually read in structs of dictionary */
 	readstructs(&argc,argv);
 
-	initlog();
+	initlog(); /* old way or new way: same name */
+
 	showEnglish = ! GetXtBoolean("showkana","Showkana");
 	CountKanji();
 	if(numberofkanji<HAVE_AT_LEAST){
